<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;

use \Geral\Entities\Empresa;
use \Geral\Util\Ambiente;
class RiscoSacadaCnabItau extends LeitorCnab{

    CONST ID_REGISTRO_HEADER = '0';
    CONST ID_REGISTRO_LOTE = '1';
    CONST ID_REGISTRO_TRANSACAO = '3';
    CONST ID_REGISTRO_TRAILLER_LOTE = '5';
    CONST ID_REGISTRO_TRAILLER = '9';

    CONST TIPO_MOVIMENTO_INCLUSAO = 0;
    CONST TIPO_MOVIMENTO_ALTERACAO = 5;
    CONST TIPO_MOVIMENTO_EXCLUSAO = 9;

    private $tpProcessamento;
    private $cdagencia;
    private $contcorr;
    private $ocorrHeader;
    private $nrlanbol = 0;
    private $cdfilial = 0;

    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {

            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $linha
     * @return bool
     */
    private function validaArquivo($linha){

        parent::setStatusArquivo(true);
        $idReg = substr($linha,7,1);
        if($this->contlinha == 1 and $idReg != self::ID_REGISTRO_HEADER ){
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            parent::setStatusArquivo(false);
        }
        if($this->contlinha > 1 and (! in_array($idReg, self::getEstruturaArquivo())
        )){
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            parent::setStatusArquivo(false);
        }
        return parent::getStatusArquivo();
    }

    public function getEstruturaArquivo(){
        return array(
                self::ID_REGISTRO_HEADER,
                self::ID_REGISTRO_LOTE,
                self::ID_REGISTRO_TRANSACAO,
                self::ID_REGISTRO_TRAILLER_LOTE,
                self::ID_REGISTRO_TRAILLER
            );
    }

    public function leArquivo($nrorg)
    {
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){
                    if (strlen($linha) > 0){
                        $this->retorno = new Retorno();

                        // Busca o Tipo de Registro do Arquivo
                        $idRegistro = substr($linha,7,1);

                        if(self::validaArquivo($linha)){
                            switch ($idRegistro) {
                                case(self::ID_REGISTRO_HEADER):
                                    self::leHeaderArquivo($linha);
                                    break;

                                case(self::ID_REGISTRO_TRANSACAO):
                                    self::leRegistroDetalhe($linha);
                                    break;

                                case(self::ID_REGISTRO_LOTE):
                                    self::leRegistroLote($linha);
                                    break;

                                case(self::ID_REGISTRO_TRAILLER_LOTE):
                                    break;
                            }
                        }
                        $this->retornos[] = $this->retorno;
                    }
                }
            }
            parent::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    // =====================================================================  //
    // ====================      REGISTRO HEADER        ====================  //
    // =====================================================================  //
    /**
     * @param $header
     */
    private function leHeaderArquivo($header){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if (substr($header, 0, 3) != '341'){
            self::gravaErros($this->contlinha, "Código do banco inválido.",true);
            parent::setStatusArquivo(false);
        }

        // CAMPO: Tipo de inscrição da empresa
        // POSIÇÃO: 18 a 18
        // PICTURE: 9(1)
        $idInsc = trim(substr($header, 17, 1));
        if (empty($idInsc)){
            self::gravaErros($this->contlinha, "Tipo de incrição da empresa não informado",true);
            parent::setStatusArquivo(false);
        }

        // CAMPO: Código Remessa/Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if(substr($header, 142, 1) != '2'){
            self::gravaErros($this->contlinha, "Arquivo de retorno inválido.", false);
            parent::setStatusArquivo(false);
        }
    }

    // =====================================================================  //
    // ====================  REGISTRO HEADER  DE LOTE   ====================  //
    // =====================================================================  //
    private function leRegistroLote($registro){
        // CAMPO: Número da agência centralizadora
        // POSIÇÃO: 53 a 57
        // PICTURE: 9(4)
        $this->cdagencia = substr($registro, 52, 4);

        // CAMPO: Conta debitada
        // POSIÇÃO: 59 a 70
        // PICTURE: 9(9)
        $this->contcorr = substr($registro, 58, 12);

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        /*$this->ocorrHeader = substr($registro, 230, 10);*/
    }

    // =====================================================================  //
    // ====================      REGISTRO DETALHE       ====================  //
    // =====================================================================  //
    private function leRegistroDetalhe($registro){
        $ocorrencia = substr($registro, 13, 1);
        switch($ocorrencia){
            case 'A': $this->segmentoA($registro);
                break;
            case 'J': $this->segmentoJ($registro);
                break;
            case 'B': $this->segmentoB($registro);
                break;
        }
        $nrlanbolpag = str_pad(trim($this->nrlanbol), 10, "0", STR_PAD_LEFT);
        $this->retorno->setCdfilial(trim($this->cdfilial));
        $this->retorno->setNrlanbolpag($nrlanbolpag);
        $this->retorno->setBoleto('BOLETO');
        $this->retorno->setCdconta($this->contcorr);
        $this->retorno->setCdagencia($this->cdagencia);
        $this->retorno->setCdlinha(str_pad($this->contlinha,'0',STR_PAD_LEFT));

        // CAMPO: Tipo de movimento   [0 - Inclusão  9 - Exclusão]
        // POSIÇÃO: 15 a 17
        // PICTURE: 9(3)
        $this->retorno->getTpmovimento(substr($registro, 14, 3));

        // CAMPO: Ocorrência
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->retorno->setOcorrencia(substr($registro, 230, 10));
    }

    public function segmentoA($registro){
        $this->cdfilial = substr($registro, 177, 4);
        $this->nrlanbol = substr($registro, 181, 10);
        $this->retorno->setNossonumero(substr($registro, 134, 15));
        $this->retorno->setDtvencimento(substr($registro, 93, 8));
        $this->retorno->setIdsitucnabp(substr($registro, 130, 8));
         // CAMPO: Data do pagamento
         // POSIÇÃO: 94 a 101
         // PICTURE: 9(8)
         $this->retorno->setDtpagamento(substr($registro, 154, 8));
         // CAMPO: Valor do pagamento
         // POSIÇÃO: 120 a 134
         // PICTURE: 9(13)V9(2)
         $vrpago = floatval(substr($registro, 162, 15));
         $this->retorno->setVrpago(round($vrpago / 100, 2));
         $nrRetorno = trim(substr($registro, 134, 15));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
        $ocorrencias = trim(substr($registro, 215, 15));
        if (empty($ocorrencias)){
            self::gravaErros($this->contlinha, 'Ocorrência não informada.', false);
            parent::setStatusArquivo(false);
        }

    }

    public function segmentoJ($registro){
        $this->cdfilial = substr($registro, 192, 4);
        $this->nrlanbol = substr($registro, 196, 10);
        $this->retorno->setNossonumero(substr($registro, 215, 15));
        $this->retorno->setDtvencimento(substr($registro, 91, 8));
        // CAMPO: Descontos
        // POSIÇÃO: 115 a 129
        // PICTURE: 9(13)V9(2)
        $vrdesconto = floatval(substr($registro, 114, 15));
        $this->retorno->setVrdesconto(round($vrdesconto / 100, 2));
        // CAMPO: Acréscimo
        // POSIÇÃO: 130 a 144
        // PICTURE: 9(13)V9(2)
        $this->retorno->setVracrescimo('0');
        // CAMPO: Data do pagamento
        // POSIÇÃO: 145 a 152
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 144, 8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 153 a 167
        // PICTURE: 9(13)V9(2)
        $vrpago = floatval(substr($registro, 152, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        // CAMPO: Nosso número
        // POSIÇÃO: 216 a 230
        // PICTURE: X(15)
        $nrRetorno = trim(substr($registro, 215, 15));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
        $ocorrencias = trim(substr($registro, 215, 15));
        if (empty($ocorrencias)){
            self::gravaErros($this->contlinha, 'Ocorrência não informada.', false);
            parent::setStatusArquivo(false);
        }
    }

    public function segmentoB($registro){
        $cdfilial = substr($registro, 174, 4);
        $nrlanbol = substr($registro, 178, 10);
        $this->retorno->setNossonumero(substr($registro, 215, 15));
        $this->retorno->setDtvencimento(substr($registro, 91, 8));
        // CAMPO: Data do pagamento
        // POSIÇÃO: 137 a 144
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 136, 8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 122 a 136
        // PICTURE: 9(13)V9(2)
        $vrpago = floatval(substr($registro, 121, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        // CAMPO: Nosso número
        // POSIÇÃO: 216 a 230
        // PICTURE: X(15)
        $nrRetorno = '';
    }
}