<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Fiscal\Entities\Notafiscal;
use Financeiro\Entities\Movfluxo;
use Financeiro\Entities\Ajbolrec;
use Financeiro\Entities\Ajctreceber;
use Financeiro\Entities\Tprecboleto;
use Financeiro\Entities\Adiantareceb;
use Teknisa\Model\Financeiro\Entities\Boletreceber;
use Teknisa\Model\Financeiro\Entities\Contareceber;
use Financeiro\Entities\Ratctreceber;
use Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use Geral\Util\Novocodigo;
use Teknisa\Libs\Util\DateUtil;
use Indeterminado\Entities\Devolucaorec;
use Setforms2\Shared\Registry;
use Teknisa\FIN\Modules\ContaReceber\Factory\ParcelamentoBoletoFactory;
use Teknisa\Libs\Util\NewCode;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;

/**
 * Class ParcelamentoBoletoService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class ParcelamentoBoletoService {

    const ID_AJ_BOL_COMP               = '4';
    const BOLETO_ALTERADO              = '3';
    const ID_CNAB_N_ENVIADO            = '00';
    const TAMANHO_NRLANBOLREC          = 10;
    const IDBOLRECALTER_DEFAULT        = '1';
    const IDFORMCADBREC_DEFAULT        = '1';
    const STATUS_BOLETO_PENDENTE       = 'A';
    const PERMITE_ALTERACAO_BOLETO     = '2';
    const PREFIXO_NOVOCODIGO_BOLREC    = 'BOLETRECEBER';
    const NAO_PERMITE_ALETRACAO_BOLETO = '1';

    /**
     * @param EntityManager $entityManager
     * @param ParcelamentoBoletoFactory $parcelamentoBoletoFactory
     */
    public function __construct(EntityManager $entityManager, ParcelamentoBoletoFactory $parcelamentoBoletoFactory, FluxoDeCaixaService $fluxoDeCaixaService, NewCode $newCode)
    {
        $this->entityManager = $entityManager;
        $this->parcelamentoBoletoFactory = $parcelamentoBoletoFactory;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
        $this->newCode = $newCode;
    }

    /**
     * Gera Parcelamento Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param array $parcelas
     * @throws \Exception
     */
    public function geraParcelamentoTitulo(
        $nrorg, $cdfilial, $nrlanbolrec, $parcelas = array()
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Variáveis de Inicialização
            $incboletos    = 0;
            $valortotal    = 0;
            $valortotalbol = 0;
            $nrnovoboleto  = '';

            // Busca Boleto Original
            /** @var $boletooriginal Boletreceber */
            $boletooriginal = $this->parcelamentoBoletoFactory->buscaBoletoReceber(
                $nrorg, $cdfilial, $nrlanbolrec
            );

            // Nº Boleto Original
            if((bool)$boletooriginal->getNrorigbolrec() === false)
                 $nrorigbolrec = $nrlanbolrec;
            else $nrorigbolrec = $boletooriginal->getNrorigbolrec();

            // Valor Boleto Original
            $vrantboleto = $boletooriginal->getVrfinalbrec();

            // Busca Associados ao de Origem
            /** @var $boletooriginal Boletreceber */
            $boletoorigem = $this->parcelamentoBoletoFactory->buscaBoletosOrigem(
                $nrorg, $cdfilial, $nrlanbolrec
            );

            // Soma Valor Total Boleto
            if($boletoorigem)
                foreach ($boletoorigem as $boleto)
                    $valortotalbol += $boleto->getVrfinalbrec();
            else $valortotalbol = $vrantboleto;

            // Soma Valor Total Parcelas
            for($i = 0;$i<count($parcelas);$i++)
                $valortotal += $parcelas[$i]["VRFINALBREC"];

            /** @var Contareceber $contaoriginal */
            $contaoriginal = $this->parcelamentoBoletoFactory->buscaContareceberBoleto(
                $nrorg, $cdfilial, $nrorigbolrec
            );

            $cdfilial_rat     = $contaoriginal->getCdfilial();
            $nrlanctonf_rat   = $contaoriginal->getNrlanctonf();
            $nrseqvenrec_rat  = $contaoriginal->getNrseqvenrec();
            $nrparcelarec_rat = $contaoriginal->getNrparcelarec();
            $vratuarec_rat    = $contaoriginal->getVratuarec();
            $nrorg_rat        = $contaoriginal->getNrorg();

            // Busca Contas Boleto Parcelado
            $contasoriginais = $this->parcelamentoBoletoFactory->buscaContaOriginalParcelamento(
                $nrorg, $cdfilial, $nrlanbolrec
            );
            
            // Percorre Parcelas
            for($i = 0;$i<count($parcelas);$i++){
                $parcela = $parcelas[$i];
                
                // Verifica se status da parcela é diferente de Baixado(B)
                if($parcela["IDSTATBREC"] != 'B'){
                    
                    // Exclui Tipos de Recebimento Parcela
                    $this->excluiTiposRecebimentoBoleto($parcela);
    
                    // Verifica Formato Data
                    $dtvenbolrec = $this->formataData($parcela['DTVENBOLREC']);
    
                    // Verifica Boleto Original
                    if($i == 0){
                        
                        // Atualiza Boleto Original
                        $boletooriginal->setVrfinalbrec($parcela['VRFINALBREC']);//
                        $boletooriginal->setDtvenbolrec($dtvenbolrec);//
                        $boletooriginal->setDslinhabarrec(null);
                        $boletooriginal->setCdbarrarec(null);
                        $boletooriginal->setCdnosnumrec(null);
                        $boletooriginal->setTxmensbolbol(null);
                        $boletooriginal->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);
                        $boletooriginal->setNrcnabrecrem(null);
                        $boletooriginal->setVrbolrec(round($parcela['VRFINALBREC'], 2));
                        $boletooriginal->setIdstatbrec(self::STATUS_BOLETO_PENDENTE);
                        $boletooriginal->setCdmotiparc($parcela['CDMOTIPARC']);
                        $boletooriginal->setNrbaixarec(null);
                        $boletooriginal->setCdfilibairec(null);
                        $boletooriginal->setNrparcbolrec(str_pad($parcela["NRPARCBOLREC"], 2, 0, STR_PAD_LEFT));
                        $boletooriginal->setNrparcbolrecvisu($parcela["NRPARCBOLRECVISU"]);
                        $boletooriginal->setNrorigbolrec($nrorigbolrec);
                        $boletooriginal->setIdimprbolbol($boletooriginal->getIdimprbolbol()?:'B');
                        $boletooriginal->setIdformcadbrec($boletooriginal->getIdformcadbrec()?:'1');
                        $this->entityManager->persist($boletooriginal);
                        
                    }else{
    
                        $incboletos++;
    
                        // Busca Boleto Receber
                        /** @var Boletreceber $boletonovo */
                        $boletonovo = $this->parcelamentoBoletoFactory->buscaBoletoReceber(
                            $parcela["NRORG"], $parcela["CDFILIAL"], $parcela["NRLANBOLREC"]
                        );
    
                        // Verifica Existência Boleto
                        if(!$boletonovo instanceof Boletreceber){
                            // Gera Novo Código Boleto
                            $novocodigo = $this->newCode->getCodeByTableName(self::PREFIXO_NOVOCODIGO_BOLREC.$parcela["CDFILIAL"], $nrorg);
    
                            // Insere Novo Boleto
                            $boletonovo = new Boletreceber();
                            $boletonovo->setNrorg($nrorg);
                            $boletonovo->setCdfilial($boletooriginal->getCdfilial());
                            $boletonovo->setNrlanbolrec($novocodigo);
                            $nrnovoboleto = $novocodigo;
                        }else $nrnovoboleto = $parcela["NRLANBOLREC"];
    
                        // Atualiza Boleto
                        $boletonovo->setDtvenbolrec($dtvenbolrec);
                        $boletonovo->setNrorigbolrec($nrorigbolrec);
                        $boletonovo->setCdcliente($boletooriginal->getCdcliente());
                        $boletonovo->setCdempresa($boletooriginal->getCdempresa());
                        $boletonovo->setCdreducont($boletooriginal->getCdreducont());
                        $boletonovo->setVrpemulbrec($boletooriginal->getVrpemulbrec());
                        $boletonovo->setVrmultabrec($boletooriginal->getVrmultabrec());
                        $boletonovo->setVrpemorbrec($boletooriginal->getVrpemorbrec());
                        $boletonovo->setVrmordiabrec($boletooriginal->getVrmordiabrec());
                        $boletonovo->setCdtpbaixarec($boletooriginal->getCdtpbaixarec());
                        $boletonovo->setCdcartcobbol($boletooriginal->getCdcartcobbol());
                        $boletonovo->setIdcartcobrec($boletooriginal->getIdcartcobrec());
                        $boletonovo->setCddiaprotbol($boletooriginal->getCddiaprotbol());
                        $boletonovo->setQtdiaprotbol($boletooriginal->getQtdiaprotbol());
                        $boletonovo->setCdins1cobbol($boletooriginal->getCdins1cobbol());
                        $boletonovo->setCdins2cobbol($boletooriginal->getCdins2cobbol());
                        $boletonovo->setIdimprbolbol($boletooriginal->getIdimprbolbol());
                        $boletonovo->setCdoperinclusao($boletooriginal->getCdoperinclusao());
                        $boletonovo->setIdformcadbrec($boletooriginal->getIdformcadbrec());
                        $boletonovo->setDtinclusao(DateUtil::getDataAtual());
                        $boletonovo->setIdstatbrec(self::STATUS_BOLETO_PENDENTE);
                        $boletonovo->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);
                        $boletonovo->setIdbolrecalter('1');
                        $boletonovo->setVrbolrec($parcela['VRFINALBREC']);
                        $boletonovo->setCdmotiparc($parcela['CDMOTIPARC']);
                        $boletonovo->setVrfinalbrec($parcela['VRFINALBREC']);
                        $boletonovo->setNrparcbolrec(str_pad($parcela["NRPARCBOLREC"], 2, 0, STR_PAD_LEFT));
                        $boletonovo->setNrparcbolrecvisu($parcela["NRPARCBOLRECVISU"]);
                        $boletonovo->setNrbaixarec(null);
                        $boletonovo->setCdbarrarec(null);
                        $boletonovo->setCdnosnumrec(null);
                        $boletonovo->setCdfilibairec(null);
                        $boletonovo->setDslinhabarrec(null);
                        $boletonovo->setIdativo('S');
    
                        $this->entityManager->persist($boletonovo);
                        $this->entityManager->flush();
                    }
    
                    $vrtotconta     = 0;
                    $contadorcontas = 0;
    
                    // Percorre Contas Boleto Parcelado
                    foreach ($contasoriginais as $conta){
                        $contadorcontas++;
    
                        $sqlmaxconta = $this->parcelamentoBoletoFactory->buscaMaxCntParcReceber(
                            $conta['NRORG'], $conta['CDFILIAL'], $conta['NRLANCTONF'], $conta['NRSEQVENREC']
                        );
    
                        $maiornrconta = $sqlmaxconta[0]['MAXNRPARCREC'];
                        $filialconta  = $conta['CDFILIAL'];
                        $nrlanctonf   = $conta['NRLANCTONF'];
                        $nrseqvenrec  = $conta['NRSEQVENREC'];
                        $vrconta      = round($parcela['VRFINALBREC'] * $conta['VRATUAREC']/ $vrantboleto, 2);
                        $vrtotconta  += $vrconta;
    
                        if(round($valortotal, 2) != round($valortotalbol,2)){
                            /** @var Notafiscal $notafiscal */
                            $notafiscal = $this->parcelamentoBoletoFactory->buscaNotafiscal(
                                $conta['NRORG'], $conta['CDFILIAL'], $conta['NRLANCTONF']
                            );
    
                            $notafiscal->setVrprodutos($valortotal);
                            $notafiscal->setVrnotafisc($valortotal);
                            $this->entityManager->persist($notafiscal);
                        }
    
                        // Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta['CDEMPRESA'], $conta['CDFIMOVFLREC'], $conta['CDCLASSFINA'],
                            $parcela['DTVENBOLREC'], $vrconta, 'E', 'E'
                        );
                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();
    
                        // Busca Rateio Conta Boleto Parcelado
                        /** @var $delrateio Ratctreceber */
                        $delrateio = Ratctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                            $filialconta, $nrlanctonf, $conta['NRSEQVENREC'],
                            $conta['NRPARCELAREC'], $nrorg
                        );
    
                        // Deleta Rateio Contas Boleto Parcelado
                        if((bool)$delrateio === true)
                            foreach ($delrateio as $rat)
                                $this->entityManager->remove($rat);
    
                        
                        // Deleta Ajustes Conta Boleto Parcelado
                        /** @var Ajctreceber $ajctreceber */
                        $ajctreceber = Ajctreceber::findByNrorgAndCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarec(
                            $conta['NRORG'], $conta['CDFILIAL'], $conta['NRLANCTONF'],
                            $conta['NRSEQVENREC'], $conta['NRPARCELAREC']
                        );
    
                        // Deleta Rateio Contas Boleto Parcelado
                        if((bool)$ajctreceber === true)
                            foreach ($ajctreceber as $delajuste){
                                $this->entityManager->remove($delajuste);
                            }
                                
                        $this->entityManager->flush();
                    
                        $new = false;
                        
                        // Verifica Boleto Original
                        if($i==0){
                            // Gera Nº Parcela
                            $nrparcelarec = str_pad($parcela["NRPARCBOLREC"], 2, 0, STR_PAD_LEFT);
                            $nrparcelarecvisu = $parcela["NRPARCBOLRECVISU"];
                            
    
                            // Atualiza Fluxo de Caixa
                            /** @var Movfluxo $movfluxo */
                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                $nrorg, $conta['CDEMPRESA'], $conta['CDFIMOVFLREC'], $conta['CDCLASSFINA'],
                                $conta['DTMOVFLUXO'], $conta['VRATUAREC'], $conta['IDMOVFLUXO'],
                                'E'
                            );
    
                            $this->entityManager->persist($movfluxo);
                            $this->entityManager->flush();
    
                            /** @var $contareceber Contareceber */
                            $contareceber = $this->parcelamentoBoletoFactory->buscaContareceber(
                                $nrorg, $filialconta, $nrlanctonf, $nrseqvenrec, $conta['NRPARCELAREC']
                            );
    
                            // Atualiza Conta
                            $contareceber->setVratuarec($vrconta);
                            $contareceber->setVrorigrec(round($vrconta - $conta['VRAJUSTE'], 2));
                            $contareceber->setDtatuavenrec($dtvenbolrec);
                            $contareceber->setIdmovfluxo('E');
                            $contareceber->setCdmotiparc($parcela['CDMOTIPARC']);
                            $contareceber->setNrparcelarec('01');
                            $contareceber->setNrparcelarecvisu('001');
                            $this->entityManager->persist($contareceber);
    
                            // Busca Rateio Conta Boleto Parcelado
                            /** @var Ratctreceber $rateio */
                            $rateios = Ratctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                                $filialconta, $nrlanctonf, $nrseqvenrec,
                                $nrparcelarec, $nrorg
                            );
                        }else{
    
                            /** @var $novaconta Contareceber */
                            $novaconta = $this->parcelamentoBoletoFactory->buscaContareceberBoleto(
                                $parcela['NRORG'], $parcela['CDFILIAL'], $parcela['NRLANBOLREC']
                            );
    
                            if(!$novaconta instanceof Contareceber) {
                                // Gera Nº Parcela
                                $nrparcelarec = str_pad($parcela["NRPARCBOLREC"], 2, 0, STR_PAD_LEFT);//str_pad($maiornrconta+1,2,'0',STR_PAD_LEFT);
                                $nrparcelarecvisu = $parcela["NRPARCBOLRECVISU"];//str_pad($maiornrconta+1,2,'0',STR_PAD_LEFT);
                                
                                // Descrição Conta Receber
                                $dscontarec_parc = isset($parcela['DSCONTAREC'])?$parcela['DSCONTAREC']:$conta['DSCONTAREC'];
    
                                // Insere Nova Conta
                                $novaconta = new Contareceber();
                                $novaconta->setCdfilial($filialconta);
                                $novaconta->setNrlanctonf($nrlanctonf);
                                $novaconta->setNrseqvenrec($nrseqvenrec);
                                $novaconta->setNrparcelarec($nrparcelarec);
                                $novaconta->setNrparcelarecvisu($nrparcelarecvisu);
                                $novaconta->setCdfilibolrec($cdfilial);
                                $novaconta->setNrlanbolrec($nrnovoboleto);
                                $novaconta->setCdmotiparc(($parcela['CDMOTIPARC']));
                                $novaconta->setCdempresa($conta['CDEMPRESA']);
                                $novaconta->setCdfimovflrec($conta['CDFIMOVFLREC']);
                                $novaconta->setCdclassfina($conta['CDCLASSFINA']);
                                $novaconta->setDtmovfluxo($dtvenbolrec);
                                $novaconta->setIdmovfluxo('E');
                                $novaconta->setQtdiasvenrec($conta['QTDIASVENREC']);
                                $novaconta->setIdstatrec($conta['IDSTATREC']);
                                $novaconta->setCdtpctreceb($conta['CDTPCTRECEB']);
                                $novaconta->setDscontarec($dscontarec_parc);
                                $novaconta->setCdfilialdup($conta['CDFILIALDUP']);
                                $novaconta->setNrlanctodup($conta['NRLANCTODUP']);
                                $novaconta->setNrorg($nrorg);
                                $novaconta->setIdcontaimposto('N');
                                $novaconta->setIdativo('S');
                                $novaconta->setDtinclusao(DateUtil::getDataAtual());
    
                                // Busca Rateio Conta Boleto Parcelado
                                /** @var Ratctreceber $rateio */
                                $rateios = Ratctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                                    $cdfilial_rat, $nrlanctonf_rat, $nrseqvenrec_rat,
                                    $nrparcelarec_rat, $nrorg_rat
                                );
    
                                $cdfilial_rat     = $filialconta;
                                $nrlanctonf_rat   = $nrlanctonf;
                                $nrseqvenrec_rat  = $nrseqvenrec;
                                $nrparcelarec_rat = $nrparcelarec;
                                $nrorg_rat        = $nrorg;
    
                                $new = true;
                            }else{
                                // Busca Rateio Conta Boleto Parcelado
                                /** @var Ratctreceber $rateio */
                                $rateios = Ratctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                                    $filialconta, $nrlanctonf, $nrseqvenrec,
                                    $nrparcelarec, $nrorg
                                );
                            }
    
                            // Verifica Diferença Soma Valor Total
                            if($contadorcontas == count($contasoriginais)){
                                if ((float) $vrtotconta != (float)$parcela['VRFINALBREC'])
                                    $vrconta +=  $parcela['VRFINALBREC'] - $vrtotconta;
                            }
    
                            $novaconta->setDtatuavenrec(self::formataData($parcela['DTVENBOLREC']));
                            $novaconta->setDtorigvenrec(self::formataData($parcela['DTVENBOLREC']));
                            $novaconta->setVratuarec($vrconta);
                            $novaconta->setVrorigrec($vrconta);
    
                            $this->entityManager->persist($novaconta);
                        }
    
                        $this->entityManager->flush();
    
                        $totalrateio = 0;
                        $contadorrateio = 0;
                        
                        // Percorre Ajustes Conta Boleto Parcelado
                        foreach ($ajctreceber as $ajuste){
                            // Insere Ajuste Conta
                            $novoajuste = new Ajctreceber();
                            $novoajuste->setCdfilial($filialconta);
                            $novoajuste->setNrlanctonf($nrlanctonf);
                            $novoajuste->setNrseqvenrec($nrseqvenrec);
                            $novoajuste->setNrparcelarec($nrparcelarec);
                            $novoajuste->setDtajusterec($ajuste->getDtajusterec());
                            $novoajuste->setCdajuctrec($ajuste->getCdajuctrec());
                            $novoajuste->setDsajusterec($ajuste->getDsajusterec());
                            $novoajuste->setVrajusterec($ajuste->getVrajusterec());
                            $novoajuste->setCdclassfina($ajuste->getCdclassfina());
                            $novoajuste->setCdemmfajcntr($ajuste->getCdemmfajcntr());
                            $novoajuste->setCdfimfajcntr($ajuste->getCdfimfajcntr());
                            $novoajuste->setDtmovfluxo($ajuste->getDtmovfluxo());
                            $novoajuste->setIdmovfluxo($ajuste->getIdmovfluxo());
                            $novoajuste->setIdorigajurec($ajuste->getIdorigajurec());
                            $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                            $novoajuste->setNrorg($nrorg);
    
                            $this->entityManager->persist($novoajuste);
                        }
    
                            
                        // Percorre Rateios Conta Boleto Parcelado
                        foreach ($rateios as $rateio){
                            $contadorrateio++;
    
                            if($new)
                                 $vrrateio = round($vrconta * ((float)$rateio->getVrratccctrec() / (float)($vratuarec_rat)) ,2);
                            else $vrrateio = round($vrconta * ((float)$rateio->getVrratccctrec() / (float)($conta['VRATUAREC'])) ,2);
                            $totalrateio += $vrrateio;
    
                            // Insere Rateio Conta
                            $novorateio = new Ratctreceber();
                            $novorateio->setCdfilial($filialconta);
                            $novorateio->setNrlanctonf($nrlanctonf);
                            $novorateio->setNrseqvenrec($nrseqvenrec);
                            $novorateio->setNrparcelarec($nrparcelarec);
                            $novorateio->setCdcentcust($rateio->getCdcentcust());
                            $novorateio->setCdfiratctrec($rateio->getCdfiratctrec());
                            $novorateio->setVrratccctrec($vrrateio);
                            $novorateio->setDtinclusao(DateUtil::getDataAtual());
                            $novorateio->setNrorg($nrorg);
    
                            // Verifica Diferença Soma Valor Total
                            if($contadorrateio == count($rateios)){
                                if((float)$vrtotconta != (float)$vrconta)
                                    $novorateio->setVrratccctrec($vrrateio-($vrconta-$totalrateio));
                            }
    
                            $this->entityManager->persist($novorateio);
                        }
    
                        $vratuarec_rat = $vrconta;
    
                        $this->entityManager->flush();
                    }
                }
            }
            
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclui Tipos de Recebimento Boleto
     * @param Boletreceber $boleto
     * @throws \Exception
     */
    public function excluiTiposRecebimentoBoleto($boleto){
        try{
            // Busca Tipos de Recebimento Boleto
            /** @var Tprecboleto $tprecboleto */
            $tprecboleto = Tprecboleto::findByNrlanbolrecAndCdfilialAndNrorg(
                $boleto["NRLANBOLREC"], $boleto["CDFILIAL"], $boleto["NRORG"]
            );
            
            // Exclui Tipos de Recebimento Boleto
            if($tprecboleto)
                foreach ($tprecboleto as $tpreceber)
                    $this->entityManager->remove($tpreceber);
    
            $this->entityManager->flush();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Deleta Parcela Títulos a Receber/Recebidos
     * @param Boletreceber $boletoant
     * @param Boletreceber $boletoatual
     * @param $qtdparcelas
     * @throws \Exception
     */
    public function DeletaParcela(
        $parcelas
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            //Valida ùltima parcela
            $boletoatual = $this->parcelamentoBoletoFactory->buscaBoletoReceber(
                $parcelas[count($parcelas)-1]['NRORG'],
                $parcelas[count($parcelas)-1]['CDFILIAL'],
                $parcelas[count($parcelas)-1]['NRLANBOLREC']
            );

            if ($boletoatual->getIdstatbrec() == 'B')
                throw new \Exception('Operação bloqueada. Não é possível excluir a parcela baixada.');
            if ($boletoatual->getIdstatbrec() == 'E')
                throw new \Exception('Operação bloqueada. Não é possível excluir a parcela emitida.');

            // Válida penúltima parcela
            $boletoant = $this->parcelamentoBoletoFactory->buscaBoletoReceber(
                $parcelas[count($parcelas)-2]['NRORG'],
                $parcelas[count($parcelas)-2]['CDFILIAL'],
                $parcelas[count($parcelas)-2]['NRLANBOLREC']
            );

            if ($boletoant->getIdstatbrec() == 'B')
                throw new \Exception('Operação bloqueada. Não é possível excluir a parcela com a anterior baixada.');
            if ($boletoant->getIdstatbrec() == 'E')
                throw new \Exception('Operação bloqueada. Não é possível excluir a parcela com a anterior emitida.');

            $qtdparcelas = count($parcelas);

            /** @var $tprecboleto Tprecboleto */
            $tprecboleto = Tprecboleto::findOneByCdfilialAndNrlanbolrecAndNrorg(
                $boletoatual->getCdfilial(), $boletoatual->getNrlanbolrec(), $boletoatual->getNrorg()
            );

            if ($tprecboleto){
                $this->entityManager->remove($tprecboleto);
                $this->entityManager->persist($tprecboleto);
            }

            // Updata o a parcela anterior com o valor do boleto a ser excluido
            $boletoant->setVrbolrec(round($boletoant->getVrbolrec(), 2) + round($boletoatual->getVrbolrec(), 2));
            $boletoant->setVrfinalbrec(round($boletoant->getVrfinalbrec(), 2) + round($boletoatual->getVrfinalbrec(), 2));
            $boletoant->setDslinhabarrec(null);
            $boletoant->setCdnosnumrec(null);
            $boletoant->setCdbarrarec(null);

            if ($qtdparcelas == 2) {
                $boletoant->setCdmotiparc('');
                $boletoant->setNrparcbolrec('00');
                $boletoant->setNrparcbolrecvisu('00');
                $boletoant->setNrorigbolrec('');
            }

            $this->entityManager->persist($boletoant);

            //Busca os ajustes da parcela a ser excluÃ­da
            /** @var Ajbolrec $ajustes */
            $ajustes = Ajbolrec::findByCdfilialAndNrlanbolrecAndNrorg($boletoatual->getCdfilial(),
                $boletoatual->getNrlanbolrec(),
                $boletoatual->getNrorg());

            /** @var $ajuste Ajbolrec */
            foreach ($ajustes as $ajuste) {

                /** @var $ajustebolant Ajbolrec */
                $ajustebolant = Ajbolrec::findOneByCdfilialAndNrlanbolrecAndCdajuctrecAndNrorg(
                    $boletoant->getCdfilial(), $boletoant->getNrlanbolrec(),
                    $ajuste->getCdajuctrec(), $boletoant->getNrorg()
                );

                // Se ja existir no boleto anteior o cÃ³digo de ajuste do boleto excluido updata o mesmo
                if ($ajustebolant) {
                    $ajustebolant->setVrajbolrec(round($ajustebolant->getVrajbolrec(), 2) + round($ajuste->getVrajbolrec(), 2));
                    $this->entityManager->persist($ajustebolant);
                } else {
                    //se nÃ£o existir insere
                    $novoajuste = new Ajbolrec();
                    $novoajuste->setNrorg($ajuste->getCdajuctrec());
                    $novoajuste->setCdfilial($boletoant->getCdfilial());
                    $novoajuste->setNrlanbolrec($boletoant->getNrlanbolrec());
                    $novoajuste->setVrajbolrec(round($ajuste->getVrajbolrec(), 2));
                    $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                    $this->entityManager->persist($novoajuste);
                }

                //Busca existÃªncia de adiantamento para a parcela ant.
                /** @var $adiantamentoAnt \Financeiro\Entities\Adiantareceb */
                $adiantamentoAnt = Adiantareceb::findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg(
                    $boletoant->getCdfilial(), $boletoant->getNrlanbolrec(), $ajuste->getCdajuctrec(), $boletoant->getNrorg()
                );

                //Busca existÃªncia de devolucao para a parcela ant.
                /** @var $devolucaoAnt Devolucaorec */
                $devolucaoAnt = Devolucaorec::findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg(
                    $boletoant->getCdfilial(), $boletoant->getNrlanbolrec(), $ajuste->getCdajuctrec(), $boletoant->getNrorg()
                );

                //Busca existÃªncia de adiantamento para a parcela atual.
                /** @var $adiantamentoAtual \Financeiro\Entities\Adiantareceb */
                $adiantamentoAtual = Adiantareceb::findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg(
                    $ajuste->getCdfilial(), $ajuste->getNrlanbolrec(), $ajuste->getCdajuctrec(), $ajuste->getNrorg()
                );

                //Busca existÃªncia de devolucao para a parcela atual.
                /** @var $devolucaoAtual Devolucaorec */
                $devolucaoAtual = Devolucaorec::findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg(
                    $ajuste->getCdfilial(), $ajuste->getNrlanbolrec(), $ajuste->getCdajuctrec(), $ajuste->getNrorg()
                );

                $this->entityManager->remove($ajuste);
                $this->entityManager->flush();

                if ($adiantamentoAtual) {

                    if ($adiantamentoAnt) {
                        $adiantamentoAnt->setVradiantarec(round($adiantamentoAnt->getVradiantarec(), 2) +
                            round($adiantamentoAtual->getVradiantarec(), 2));
                        $this->entityManager->persist($adiantamentoAnt);
                    } else {
                        $novoadiantamento = new Adiantareceb();
                        $novoadiantamento->setNrorg($adiantamentoAtual->getNrorg());
                        $novoadiantamento->setCdfilial($adiantamentoAtual->getCdfilial());
                        $novoadiantamento->setNrlanctonf($adiantamentoAtual->getNrlanctonf());
                        $novoadiantamento->setNrseqvenrec($adiantamentoAtual->getNrseqvenrec());
                        $novoadiantamento->setNrparcelarec($adiantamentoAtual->getNrparcelarec());
                        $novoadiantamento->setCdfiliajbolr($boletoant->getCdfilial());
                        $novoadiantamento->setNrlanbolrec($boletoant->getNrlanbolrec());
                        $novoadiantamento->setCdajuctrec($adiantamentoAtual->getCdajuctrec());
                        $novoadiantamento->setDtinclusao(DateUtil::getDataAtual());
                        $this->entityManager->persist($novoadiantamento);
                    }
                }
                if ($devolucaoAtual) {
                    $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg(
                        $devolucaoAtual->getCdfilial(), $devolucaoAtual->getNrlanctonf(), $devolucaoAtual->getNrorg()
                    );

                    if ($devolucaoAnt) {
                        /** @var $notafiscal Notafiscal */

                        $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl() + $devolucaoAtual->getVrdevolucaorec());
                        $notafiscal->setCdfilial($devolucaoAtual->getCdfilial());
                        $notafiscal->setNrlanctonf($devolucaoAtual->getNrlanctonf());
                        $this->entityManager->persist($notafiscal);

                        $devolucaoAnt->setVrdevolucaorec(round($devolucaoAnt->getVrdevolucaorec() + $devolucaoAtual->getVrdevolucaorec(), 2));
                        $this->entityManager->persist($devolucaoAnt);
                    } else {
                        $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl() + $devolucaoAtual->getVrdevolucaorec());
                        $notafiscal->setCdfilial($devolucaoAtual->getCdfilial());
                        $notafiscal->setNrlanctonf($devolucaoAtual->getNrlanctonf());
                        $this->entityManager->persist($notafiscal);

                        $novadevolucao = new Devolucaorec();
                        $novadevolucao->setNrorg($boletoant->getNrorg());
                        $novadevolucao->setNrlanbolrec($boletoant->getNrlanbolrec());
                        $novadevolucao->setCdajuctrec($devolucaoAtual->getCdajuctrec());
                        $novadevolucao->setCdfilial($devolucaoAtual->getCdfilial());
                        $novadevolucao->setNrlanctonf($devolucaoAtual->getNrlanctonf());
                        $novadevolucao->setVrdevolucaorec($devolucaoAtual->getVrdevolucaorec());
                        $novadevolucao->setDtinclusao(DateUtil::getDataAtual());
                        $this->entityManager->persist($novadevolucao);
                    }

                }
            }

            //Busca contas do boleto excluido
            $contasreceber = $this->parcelamentoBoletoFactory->buscaContasreceberBoleto(
                $boletoatual->getNrorg(), $boletoatual->getCdfilial(), $boletoatual->getNrlanbolrec()
            );

            $primconta       = true;
            $countconta      = 0;
            $contasoriginais = [];

            /** @var $conta Contareceber */
            foreach ($contasreceber as $conta) {
                if ($primconta) {
                    $contasoriginais = $this->parcelamentoBoletoFactory->buscaContaOriginalParcelamento(
                        $boletoant->getNrorg(), $boletoant->getCdfilial(), $boletoant->getNrlanbolrec()
                    );
                    $primconta = false;
                } else $countconta++;

                $ajustesconta = $this->parcelamentoBoletoFactory->buscaAjusteConta(
                    $conta->getNrorg(), $conta->getCdfilibolrec(), $conta->getNrlanbolrec()
                );
                
                foreach ($ajustesconta as $ajusteconta) {
                    
                    $ajctreceber = Ajctreceber::findOneByNrorgAndCdfilialAndAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndCdajuctrecAndDtajusterec(
                        $ajusteconta['NRORG'], $ajusteconta['CDFILIAL'], $ajusteconta['NRLANCTONF'],
                        $ajusteconta['NRSEQVENREC'], $contasoriginais[$countconta]['NRPARCELAREC'],
                        $ajusteconta['CDAJUCTREC'], $ajusteconta['DTAJUSTEREC']
                    );

                    if (!$ajctreceber) {
                        $novoajuste = new Ajctreceber();
                        $novoajuste->setCdfilial($ajusteconta['CDFILIAL']);
                        $novoajuste->setNrlanctonf($ajusteconta['NRLANCTONF']);
                        $novoajuste->setDtajusterec($ajusteconta['DTAJUSTEREC']);
                        $novoajuste->setCdajuctrec($ajusteconta['CDAJUCTREC']);
                        $novoajuste->setDsajusterec($ajusteconta['DSAJUSTEREC']);
                        $novoajuste->setNrseqvenrec($ajusteconta['NRSEQVENREC']);
                        $novoajuste->setNrparcelarec($contasoriginais[$countconta]['NRPARCELAREC']);
                        $novoajuste->setCdclassfina($ajusteconta['CDCLASSFINA']);
                        $novoajuste->setCdemmfajcntr($ajusteconta['CDEMMFAJCNTR']);
                        $novoajuste->setCdfimfajcntr($ajusteconta['CDFIMFAJCNTR']);
                        $novoajuste->setDtmovfluxo($ajusteconta['DTMOVFLUXO']);
                        $novoajuste->setIdmovfluxo($ajusteconta['IDMOVFLUXO']);
                        $novoajuste->setVrajusterec($ajusteconta['VRAJUSTEREC']);
                        $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                        $this->entityManager->persist($novoajuste);
                    }

                    $delajctreceber = Ajctreceber::findOneByNrorgAndCdfilialAndAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndCdajuctrecAndDtajusterec(
                        $ajusteconta['NRORG'], $ajusteconta['CDFILIAL'], $ajusteconta['NRLANCTONF'],
                        $ajusteconta['NRSEQVENREC'], $ajusteconta['NRPARCELAREC'],
                        $ajusteconta['CDAJUCTREC'], $ajusteconta['DTAJUSTEREC']
                    );

                    $this->entityManager->remove($delajctreceber);
                }

                $contaoriginal = $this->parcelamentoBoletoFactory->buscaContareceber(
                    $contasoriginais[$countconta]['NRORG'], $contasoriginais[$countconta]['CDFILIAL'], 
                    $contasoriginais[$countconta]['NRLANCTONF'], $contasoriginais[$countconta]['NRSEQVENREC'],
                    $contasoriginais[$countconta]['NRPARCELAREC']
                );

                $valorContaOld = round($contasoriginais[$countconta]['VRATUAREC'], 2);

                $contaoriginal->setVratuarec(round($contasoriginais[$countconta]['VRATUAREC'], 2) + round($conta->getvratuarec(), 2));
                $contaoriginal->setVrorigrec(round($contasoriginais[$countconta]['VRORIGREC'], 2) + round($conta->getvrorigrec(), 2));

                $this->entityManager->persist($contaoriginal);

                // Gera Rateio Conta a Receber
                self::geraRateioContaRec(
                    $contasoriginais[$countconta]['NRORG'], $contasoriginais[$countconta]['CDFILIAL'], $contasoriginais[$countconta]['NRLANCTONF'],
                    $contasoriginais[$countconta]['NRSEQVENREC'], $contasoriginais[$countconta]['NRPARCELAREC'],
                    round($contasoriginais[$countconta]['VRATUAREC'], 2) + round($conta->getVratuarec(), 2),
                    $valorContaOld
                );

                $delconta = $this->parcelamentoBoletoFactory->buscaContareceber(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(), 
                    $conta->getNrseqvenrec(), $conta->getNrparcelarec()
                );

                $this->entityManager->remove($delconta);
            }
            $this->entityManager->flush();

            // Exclui parcela
            $this->entityManager->remove($boletoatual);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Rateio Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $valornovo
     * @param $valorantigo
     * @throws \Exception
     */
    private function geraRateioContaRec(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $valornovo, $valorantigo
    ){
        try {

            // Variáveis Default
            $contadorrateio = 0;
            $totalrateio    = 0;

            // Busca Valor Rateios Conta a Receber
            $rateios = $this->parcelamentoBoletoFactory->buscaValorRateioConta(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec
            );

            // Busca Rateios Conta a Receber
            /** @var Ratctreceber $delrateio */
            $delrateio = $this->parcelamentoBoletoFactory->buscaRatCtReceber(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec
            );

            // Exclui Rateios Existentes
            if ($delrateio)
                foreach ($delrateio as $rat)
                    $this->entityManager->remove($rat);

            $this->entityManager->flush();

            /** @var Ratctreceber $rateio */
            foreach ($rateios as $rateio) {

                // Calcula Valor do Rateio Conta a Receber
                $valorrateio = round($valornovo * ($rateio['VRRATCCCTREC'] / $valorantigo), 2);
                $totalrateio += $valorrateio;

                /** @var Ratctreceber $novorateio */
                $novorateio = $this->parcelamentoBoletoFactory->insereRatctreceber(
                    $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec,
                    $rateio['CDCENTCUST'], $rateio['CDFIRATCTREC'], $valorrateio
                );

                $contadorrateio++;

                // Verifica se existe diferença no valor do rateio e da conta
                if ($contadorrateio == count($rateios))
                    if ((float)$totalrateio != (float)$valornovo)
                        $novorateio->setVrratccctrec($valorrateio - ($valornovo - $totalrateio));

                $this->entityManager->persist($novorateio);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}